/*
 * Copyright © 2021 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel22-tictactoe-core.
 *
 * ipspiel22-tictactoe-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel22-tictactoe-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel22-tictactoe-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel22.tictactoe.core.domain;

import de.fhdw.gaming.core.domain.GameBuilder;
import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.core.domain.ObserverFactoryProvider;

/**
 * A builder which allows to create a Tic Tac Toe game.
 */
public interface TicTacToeGameBuilder extends GameBuilder {

    /**
     * The default number of rows (and columns) of a Tic Tac Toe board.
     */
    int DEFAULT_BOARD_SIZE = 3;

    /**
     * Creates an {@link TicTacToePlayerBuilder} which allows to create and add a player to the game together with her
     * strategy.
     */
    TicTacToePlayerBuilder createPlayerBuilder();

    /**
     * Adds a player and her corresponding strategy.
     *
     * @param player   The player.
     * @param strategy The player's strategy.
     * @throws GameException if adding the player is not allowed by the rules of the game.
     */
    TicTacToeGameBuilder addPlayer(TicTacToePlayer player, TicTacToeStrategy strategy) throws GameException;

    /**
     * Changes the number of rows (and columns) of the board.
     * <p>
     * If this operation is called multiple times, only the last board size will be retained. If not called, the default
     * size of {@link #DEFAULT_BOARD_SIZE} rows and columns is used.
     *
     * @param newBoardSize The new number of rows (and columns) of the board.
     * @return {@code this}
     */
    TicTacToeGameBuilder changeBoardSize(int newBoardSize);

    /**
     * Changes the {@link ObserverFactoryProvider}.
     *
     * @param newObserverFactoryProvider The new {@link ObserverFactoryProvider}.
     * @return {@code this}
     */
    TicTacToeGameBuilder changeObserverFactoryProvider(ObserverFactoryProvider newObserverFactoryProvider);

    @Override
    TicTacToeGame build(int id) throws GameException, InterruptedException;
}
